@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2013-2024, The PurpleI2P Project
+* Copyright (c) 2013-2025, The PurpleI2P Project
 *
 * This file is part of Purple i2pd project and licensed under BSD3
 *
@@ -29,6 +29,8 @@ namespace i2p
 namespace client
 {
 	const size_t SAM_SOCKET_BUFFER_SIZE = 8192;
+	const size_t SAM_STREAM_BUFFER_SIZE = 16384;
+	const size_t SAM_STREAM_MAX_SEND_BUFFER_SIZE = 8*SAM_SOCKET_BUFFER_SIZE;
 	const int SAM_SOCKET_CONNECTION_MAX_IDLE = 3600; // in seconds
 	const int SAM_SESSION_READINESS_CHECK_INTERVAL = 3; // in seconds
 	const size_t SAM_SESSION_MAX_ACCEPT_QUEUE_SIZE = 50;
@@ -170,12 +172,13 @@ namespace client
 			Socket_t m_Socket;
 			boost::asio::deadline_timer m_Timer;
 			char m_Buffer[SAM_SOCKET_BUFFER_SIZE + 1];
-			size_t m_BufferOffset;
-			uint8_t m_StreamBuffer[SAM_SOCKET_BUFFER_SIZE];
+			size_t m_BufferOffset; // for session only
+			uint8_t m_StreamBuffer[SAM_STREAM_BUFFER_SIZE];
 			SAMSocketType m_SocketType;
 			std::string m_ID; // nickname
 			bool m_IsSilent;
 			bool m_IsAccepting; // for eSAMSocketTypeAcceptor only
+			bool m_IsReceiving; // for eSAMSocketTypeStream only
 			std::shared_ptr<i2p::stream::Stream> m_Stream;
 	};
 
@@ -246,7 +249,7 @@ namespace client
 			void Start ();
 			void Stop ();
 
-			boost::asio::io_service& GetService () { return GetIOService (); };
+			auto& GetService () { return GetIOService (); };
 			std::shared_ptr<SAMSession> CreateSession (const std::string& id, SAMSessionType type, const std::string& destination, // empty string means transient
 				const std::map<std::string, std::string> * params);
 			bool AddSession (std::shared_ptr<SAMSession> session);
@@ -271,6 +274,10 @@ namespace client
 			void ReceiveDatagram ();
 			void HandleReceivedDatagram (const boost::system::error_code& ecode, std::size_t bytes_transferred);
 
+			void ScheduleSessionCleanupTimer (std::shared_ptr<SAMSession> session);
+			void HandleSessionCleanupTimer (const boost::system::error_code& ecode,
+				std::shared_ptr<SAMSession> session, std::shared_ptr<boost::asio::deadline_timer> timer);	                                
+			
 		private:
 
 			bool m_IsSingleThread;
